Index: vtfontcvt.c
===================================================================
--- vtfontcvt.c	(版本 315489)
+++ vtfontcvt.c	(工作副本)
@@ -109,7 +109,7 @@
 static int
 add_mapping(struct glyph *gl, unsigned int c, unsigned int map_idx)
 {
-	struct mapping *mp;
+	struct mapping *mp, *mp_temp = NULL;
 	struct mapping_list *ml;
 
 	mapping_total++;
@@ -120,10 +120,16 @@
 	mp->m_length = 0;
 
 	ml = &maps[map_idx];
-	if (TAILQ_LAST(ml, mapping_list) != NULL &&
-	    TAILQ_LAST(ml, mapping_list)->m_char >= c)
-		errx(1, "Bad ordering at character %u", c);
-	TAILQ_INSERT_TAIL(ml, mp, m_list);
+	TAILQ_FOREACH(mp_temp, ml, m_list) {
+		if (mp_temp->m_char >= c)
+			break;
+	}
+	if (mp_temp == NULL)
+		TAILQ_INSERT_TAIL(ml, mp, m_list);
+    else if (mp_temp->m_char == c)
+        return (0);
+	else
+		TAILQ_INSERT_BEFORE(mp_temp, mp, m_list);
 
 	map_count[map_idx]++;
 	mapping_unique++;
@@ -217,13 +223,13 @@
 	uint8_t *p;
 	unsigned int i, subline;
 
-	if (dwidth != width && dwidth != width * 2)
-		errx(1, "Bitmap with unsupported width %u!", dwidth);
+	if (dwidth > width * 2)
+		errx(1, "Character too wide %u!", dwidth);
 
 	/* Move pixel data right to simplify splitting double characters. */
 	line >>= (howmany(dwidth, 8) * 8) - dwidth;
 
-	for (i = dwidth / width; i > 0; i--) {
+	for (i = howmany(dwidth, width); i > 0; i--) {
 		p = (i == 2) ? right : left;
 
 		subline = line & ((1 << width) - 1);
@@ -251,10 +257,16 @@
 	size_t length;
 	uint8_t bytes[wbytes * height], bytes_r[wbytes * height];
 	unsigned int curchar = 0, dwidth = 0, i, line;
+	int bb_x = 0, bb_y = 0, bb_w = width, bb_h = height, line_y;
+	int font_w = width, font_h = height, font_x = 0, font_y = 0;
 
 	while ((ln = fgetln(fp, &length)) != NULL) {
 		ln[length - 1] = '\0';
 
+		if (strncmp(ln, "FONTBOUNDINGBOX ", 16) == 0) {
+			sscanf(ln + 16, "%d %d %d %d", &font_w, &font_h, &font_x, &font_y);
+		}
+
 		if (strncmp(ln, "ENCODING ", 9) == 0) {
 			curchar = atoi(ln + 9);
 		}
@@ -263,20 +275,38 @@
 			dwidth = atoi(ln + 7);
 		}
 
+		if (strncmp(ln, "BBX ", 4) == 0) {
+			sscanf(ln + 4, "%d %d %d %d", &bb_w, &bb_h, &bb_x, &bb_y);
+		}
+
 		if (strncmp(ln, "BITMAP", 6) == 0 &&
 		    (ln[6] == ' ' || ln[6] == '\0')) {
-			for (i = 0; i < height; i++) {
+			bzero(bytes, sizeof(bytes));
+			bzero(bytes_r, sizeof(bytes_r));
+
+			for (i = 0; ; i++) {
 				if ((ln = fgetln(fp, &length)) == NULL)
 					errx(1, "Unexpected EOF!");
+				if (strncmp(ln, "ENDCHAR", 7) == 0) {
+					break;
+				}
+				line_y = height + font_y - bb_h - bb_y + i;
+				if ((line_y < 0) || ((unsigned int)line_y >= height)) {
+					continue;
+				}
+
 				ln[length - 1] = '\0';
 				sscanf(ln, "%x", &line);
-				if (parse_bitmap_line(bytes + i * wbytes,
-				     bytes_r + i * wbytes, line, dwidth) != 0)
+
+				line = line >> (howmany(bb_w, 8) * 8 - bb_w)
+				    << (howmany(dwidth, 8) * 8 - bb_x - bb_w);
+				if (parse_bitmap_line(bytes + line_y * wbytes,
+				     bytes_r + line_y * wbytes, line, dwidth) != 0)
 					return (1);
 			}
 
 			if (add_char(curchar, map_idx, bytes,
-			    dwidth == width * 2 ? bytes_r : NULL) != 0)
+			    dwidth > width ? bytes_r : NULL) != 0)
 				return (1);
 		}
 	}
@@ -353,23 +383,29 @@
 }
 
 static int
-parse_file(const char *filename, unsigned int map_idx)
+parse_file(char *filenames, unsigned int map_idx)
 {
 	FILE *fp;
 	size_t len;
 	int rv;
+	char *filename;
 
-	fp = fopen(filename, "r");
-	if (fp == NULL) {
-		perror(filename);
-		return (1);
+	while ((filename = strsep(&filenames, ":")) != NULL) {
+		fp = fopen(filename, "r");
+		if (fp == NULL) {
+			perror(filename);
+			return (1);
+		}
+		len = strlen(filename);
+		if (len > 4 && strcasecmp(filename + len - 4, ".hex") == 0)
+			rv = parse_hex(fp, map_idx);
+		else
+			rv = parse_bdf(fp, map_idx);
+		fclose(fp);
+		if (rv != 0) {
+			return rv;
+		}
 	}
-	len = strlen(filename);
-	if (len > 4 && strcasecmp(filename + len - 4, ".hex") == 0)
-		rv = parse_hex(fp, map_idx);
-	else
-		rv = parse_bdf(fp, map_idx);
-	fclose(fp);
 	return (rv);
 }
 
